/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* Program:                                                            */
/* --------                                                            */
/*                                                                     */
/* galaxy.c                                                            */
/*                                                                     */
/*                                                                     */
/* Version:                                                            */
/* --------                                                            */
/*                                                                     */
/* First version: February 2000                                        */
/* Latest update: 2003-02-18                                           */
/*                                                                     */
/*                                                                     */
/* Author of the program:                                              */
/* ----------------------                                              */
/*                                                                     */
/* Stefan Spaennare, Lund, Sweden                                      */
/*                                                                     */
/* E-mail: stefan@spaennare.se                                         */
/*                                                                     */
/* Web: http://www.spaennare.se/index.html                             */
/* Web: http://www.spaennare.se/ssnbody.html                           */
/*                                                                     */
/* If you have problems with the program please let me know!           */
/*                                                                     */
/*                                                                     */
/* Documentation:                                                      */
/* --------------                                                      */
/*                                                                     */
/* For a detailed description of the ssnbody nbody package see the     */
/* files ssnbody.pdf and ssnbody.ps on the ssnbody web page.           */
/*                                                                     */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/



#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double pi=3.14159265358979323;


void galaxy(r1,nr1,m1,mf1,x1,y1,vx1,vy1,gg,outfile)
double r1;
long nr1;
double m1,mf1,x1,y1,vx1,vy1,gg;
FILE *outfile;
{

  long i,j,k,n,s,ss2,ss,nk,nm;

  double rr,v,alfa,x,y,z;
  double r2,dx,dy,dz,r,r3;
  double eps,ri,ro,aa,msum;
  double ax,ay,az,aaa,temp,mm;
  double mmm,dr1;

  double *xx1,*yy1,*zz1;
  double *vxx1,*vyy1,*vzz1;
  double *mm0;

  double *mfac;

  eps=0.001;

  dr1=r1/(double)(nr1-1);

  nm=(long)(r1/dr1+0.5);

  mfac=(double *)calloc(nm+3,sizeof(double));

  mfac[1]=1.0;

  rr=dr1;

  aa=0.25*dr1*dr1;

  n=1;
  k=2;

  while (rr <= (r1+eps)) {

      ro=rr+0.5*dr1;
      ri=rr-0.5*dr1;

      s=(long)((ro*ro-ri*ri)/aa+eps);

      mfac[k]=exp(-mf1*sqrt(sqrt(rr/r1))-1.0);

      n=n+s;

      rr=rr+dr1;

      k++;

  } /* while */

  nk=k-1;

  msum=0.0;

  for (i=1; i<=nk; i++) {

     msum=msum+mfac[i];

  } /* for i */

  printf("%6d\n",n);

  printf("%6d %18.15f\n",nk,msum);

  xx1=(double *)calloc(n+1,sizeof(double));
  yy1=(double *)calloc(n+1,sizeof(double));
  zz1=(double *)calloc(n+1,sizeof(double));
  vxx1=(double *)calloc(n+1,sizeof(double));
  vyy1=(double *)calloc(n+1,sizeof(double));
  vzz1=(double *)calloc(n+1,sizeof(double));
  mm0=(double *)calloc(n+1,sizeof(double));

  xx1[1]=0.0+x1;
  yy1[1]=0.0+y1;
  zz1[1]=0.0;
  vxx1[1]=0.0+vx1;
  vyy1[1]=0.0+vy1;
  vzz1[1]=0.0;
  mm0[1]=m1*mfac[1]/msum;

  printf("%6d %6d %18.8f %18.8f\n",1,1,mm0[1],mm0[1]);

  ss=1;
  ss2=ss;

  rr=dr1;
  k=2;

  while (rr <= (r1+eps)) {

      ro=rr+0.5*dr1;
      ri=rr-0.5*dr1;

      s=(long)((ro*ro-ri*ri)/aa+eps);

      ss=ss+s;

      alfa=0.0;

      mmm=m1*mfac[k]/(msum*(double)(s));

      for (i=ss2+1; i<=ss; i++) {

         xx1[i]=rr*cos(alfa)+x1;
         yy1[i]=rr*sin(alfa)+y1;
         zz1[i]=0.0;

         mm0[i]=mmm;

         alfa=alfa+2.0*pi/(double)(s);

      } /* for i */

      printf("%6d %6d %18.8f %18.8f\n",k,s,mmm,(double)(s)*mmm);

      ss2=ss;

      rr=rr+dr1;

      k++;

  } /* while */


  ss=1;
  ss2=ss;

  rr=dr1;

while (rr <= (r1+eps)) {

      ro=rr+0.5*dr1;
      ri=rr-0.5*dr1;

      s=(long)((ro*ro-ri*ri)/aa+eps);

      ss=ss+s;

alfa=0.0;

for (j=ss2+1; j<=ss; j++) {

  x=xx1[j];
  y=yy1[j];
  z=zz1[j];

  ax=0.0;
  ay=0.0;
  az=0.0;

  for (i=1; i<=n; i++) {

    if (i != j) {

       dx=x-xx1[i];
       dy=y-yy1[i];
       dz=z-zz1[i];
       mm=mm0[i];

       r2=dx*dx+dy*dy+dz*dz;
       r=sqrt(r2);
       r3=1.0/(r*r2);

       temp=gg*mm*r3;

       ax=ax+temp*dx;
       ay=ay+temp*dy;
       az=az+temp*dz;

    } /* if */

  } /* for i */


  aaa=sqrt(ax*ax+ay*ay+az*az);

  v=sqrt(aaa*rr);

  vxx1[j]=-v*sin(alfa)+vx1;
  vyy1[j]=v*cos(alfa)+vy1;
  vzz1[j]=0.0;

  alfa=alfa+2.0*pi/(double)(s);

} /* for j */

  ss2=ss;

  rr=rr+dr1;

} /* while */


msum=0.0;

for (i=1; i<=n; i++) {

   fprintf(outfile,"%25.17le %25.17le %25.17le %25.17le %25.17le %25.17le %25.17le\n",
          xx1[i],yy1[i],zz1[i],vxx1[i],vyy1[i],vzz1[i],mm0[i]);

   msum=msum+mm0[i];

} /* for i */


printf("%18.8f\n",msum);
printf("\n");

free(xx1);
free(yy1);
free(zz1);
free(vxx1);
free(vyy1);
free(vzz1);
free(mm0);

free(mfac);

} /* galaxy */


int main(argc,argv) 
int argc;
char *argv[];

{
   
   FILE *infile,*outfile;

   long nr1;

   double r1,m1,mf1,x1,y1,vx1,vy1;
   double gg;

   if (argc != 3) {
      printf("Usage: %s infile outfile\n",argv[0]);
      printf("\n");
      printf("Infile parameters:\n");
      printf("       gr\n");
      printf("       r1r nr1i m1r mf1r x1r y1r vx1r vy1r\n");
      printf("       r2r nr2i m2r mf2r x2r y2r vx2r vy2r\n");
      printf("                     ...\n");
      printf("       rnr nrni mnr mfnr xnr ynr vxnr vynr\n");
      exit(0);
   } /* if */

   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile = fopen(argv[2],"w");

   fscanf(infile,"%lf",&gg);

   while (fscanf(infile,"%lf %d %lf %lf %lf %lf %lf %lf",
                 &r1,&nr1,&m1,&mf1,&x1,&y1,&vx1,&vy1) != EOF) {

      galaxy(r1,nr1,m1,mf1,x1,y1,vx1,vy1,gg,outfile);

   } /* while */

   fclose(infile);
   fclose(outfile);

} /* End */
